@@ -98,8 +98,6 @@
 #define CLUTTER_TABLE_CHILD(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLUTTER_TYPE_TABLE_CHILD, ClutterTableChild))
 #define CLUTTER_IS_TABLE_CHILD(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CLUTTER_TYPE_TABLE_CHILD))
 
-#define CLUTTER_TABLE_LAYOUT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_TABLE_LAYOUT, ClutterTableLayoutPrivate))
-
 typedef struct _ClutterTableChild         ClutterTableChild;
 typedef struct _ClutterLayoutMetaClass    ClutterTableChildClass;
 
@@ -184,13 +182,9 @@ enum
 
 GType clutter_table_child_get_type (void);
 
-G_DEFINE_TYPE (ClutterTableChild,
-               clutter_table_child,
-               CLUTTER_TYPE_LAYOUT_META);
+G_DEFINE_TYPE (ClutterTableChild, clutter_table_child, CLUTTER_TYPE_LAYOUT_META)
 
-G_DEFINE_TYPE (ClutterTableLayout,
-               clutter_table_layout,
-               CLUTTER_TYPE_LAYOUT_MANAGER);
+G_DEFINE_TYPE_WITH_PRIVATE (ClutterTableLayout, clutter_table_layout, CLUTTER_TYPE_LAYOUT_MANAGER)
 
 /*
  * ClutterBoxChild
@@ -574,28 +568,28 @@ clutter_table_child_class_init (ClutterTableChildClass *klass)
   pspec = g_param_spec_boolean ("x-expand",
                                 P_("Horizontal Expand"),
                                 P_("Allocate extra space for the child in horizontal axis"),
-                                TRUE,
+                                FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_X_EXPAND, pspec);
 
   pspec = g_param_spec_boolean ("y-expand",
                                 P_("Vertical Expand"),
                                 P_("Allocate extra space for the child in vertical axis"),
-                                TRUE,
+                                FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_Y_EXPAND, pspec);
 
   pspec = g_param_spec_boolean ("x-fill",
                                 P_("Horizontal Fill"),
                                 P_("Whether the child should receive priority when the container is allocating spare space on the horizontal axis"),
-                                TRUE,
+                                FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_X_FILL, pspec);
 
   pspec = g_param_spec_boolean ("y-fill",
                                 P_("Vertical Fill"),
                                 P_("Whether the child should receive priority when the container is allocating spare space on the vertical axis"),
-                                TRUE,
+                                FALSE,
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_CHILD_Y_FILL, pspec);
 
@@ -889,8 +883,8 @@ calculate_col_widths (ClutterTableLayout *self,
             n_expand++;
         }
 
-      pref_width += priv->col_spacing * MAX (priv->visible_cols - 1, 0);
-      min_width += priv->col_spacing * MAX (priv->visible_cols - 1, 0);
+      pref_width += priv->col_spacing * (priv->n_cols - 1);
+      min_width += priv->col_spacing * (priv->n_cols - 1);
 
       if (for_width <= min_width)
         {
@@ -1179,8 +1173,8 @@ calculate_row_heights (ClutterTableLayout *self,
             n_expand++;
         }
 
-      pref_height += priv->row_spacing * MAX (priv->visible_rows - 1, 0);
-      min_height += priv->row_spacing * MAX (priv->visible_rows - 1, 0);
+      pref_height += priv->row_spacing * (priv->n_rows - 1);
+      min_height += priv->row_spacing * (priv->n_rows - 1);
 
       if (for_height <= min_height)
         {
@@ -1297,7 +1291,7 @@ clutter_table_layout_get_preferred_width (ClutterLayoutManager *layout,
   calculate_table_dimensions (self, container, -1, for_height);
   columns = (DimensionData *) (void *) priv->columns->data;
 
-  total_min_width = MAX ((priv->visible_cols - 1) * (float) priv->col_spacing, 0);
+  total_min_width = (priv->visible_cols - 1) * (float) priv->col_spacing;
   total_pref_width = total_min_width;
 
   for (i = 0; i < priv->n_cols; i++)
@@ -1337,7 +1331,7 @@ clutter_table_layout_get_preferred_height (ClutterLayoutManager *layout,
   calculate_table_dimensions (self, container, for_width, -1);
   rows = (DimensionData *) (void *) priv->rows->data;
 
-  total_min_height = MAX ((priv->visible_rows - 1) * (float) priv->row_spacing, 0);
+  total_min_height = (priv->visible_rows - 1) * (float) priv->row_spacing;
   total_pref_height = total_min_height;
 
   for (i = 0; i < self->priv->n_rows; i++)
@@ -1624,8 +1618,6 @@ clutter_table_layout_class_init (ClutterTableLayoutClass *klass)
   layout_class->get_child_meta_type =
     clutter_table_layout_get_child_meta_type;
 
-  g_type_class_add_private (klass, sizeof (ClutterTableLayoutPrivate));
-
   /**
    * ClutterTableLayout:column-spacing:
    *
@@ -1731,7 +1723,7 @@ clutter_table_layout_init (ClutterTableLayout *layout)
 {
   ClutterTableLayoutPrivate *priv;
 
-  layout->priv = priv = CLUTTER_TABLE_LAYOUT_GET_PRIVATE (layout);
+  layout->priv = priv = clutter_table_layout_get_instance_private (layout);
 
   priv->row_spacing = 0;
   priv->col_spacing = 0;
