@@ -46,7 +46,6 @@
 #include "clutter-private.h"
 #include "clutter-profile.h"
 #include "clutter-stage-private.h"
-#include "clutter-util.h"
 
 static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
 
@@ -72,28 +71,53 @@ clutter_stage_cogl_unrealize (ClutterStageWindow *stage_window)
 
   if (stage_cogl->onscreen != NULL)
     {
+      cogl_onscreen_remove_frame_callback (stage_cogl->onscreen,
+                                           stage_cogl->frame_closure);
+      stage_cogl->frame_closure = NULL;
+
       cogl_object_unref (stage_cogl->onscreen);
       stage_cogl->onscreen = NULL;
     }
 }
 
 static void
-handle_swap_complete_cb (CoglFramebuffer *framebuffer,
-                         void *user_data)
+frame_cb (CoglOnscreen  *onscreen,
+          CoglFrameEvent event,
+          CoglFrameInfo *info,
+          void          *user_data)
 {
   ClutterStageCogl *stage_cogl = user_data;
 
-  /* Early versions of the swap_event implementation in Mesa
-   * deliver BufferSwapComplete event when not selected for,
-   * so if we get a swap event we aren't expecting, just ignore it.
-   *
-   * https://bugs.freedesktop.org/show_bug.cgi?id=27962
-   *
-   * FIXME: This issue can be hidden inside Cogl so we shouldn't
-   * need to care about this bug here.
-   */
-  if (stage_cogl->pending_swaps > 0)
-    stage_cogl->pending_swaps--;
+  if (event == COGL_FRAME_EVENT_SYNC)
+    {
+      /* Early versions of the swap_event implementation in Mesa
+       * deliver BufferSwapComplete event when not selected for,
+       * so if we get a swap event we aren't expecting, just ignore it.
+       *
+       * https://bugs.freedesktop.org/show_bug.cgi?id=27962
+       *
+       * FIXME: This issue can be hidden inside Cogl so we shouldn't
+       * need to care about this bug here.
+       */
+      if (stage_cogl->pending_swaps > 0)
+        stage_cogl->pending_swaps--;
+    }
+  else if (event == COGL_FRAME_EVENT_COMPLETE)
+    {
+      gint64 presentation_time_cogl = cogl_frame_info_get_presentation_time (info);
+
+      if (presentation_time_cogl != 0)
+        {
+          CoglContext *context = cogl_framebuffer_get_context (COGL_FRAMEBUFFER (onscreen));
+          gint64 current_time_cogl = cogl_get_clock_time (context);
+          gint64 now = g_get_monotonic_time ();
+
+          stage_cogl->last_presentation_time =
+            now + (presentation_time_cogl - current_time_cogl) / 1000;
+        }
+
+      stage_cogl->refresh_rate = cogl_frame_info_get_refresh_rate (info);
+    }
 }
 
 static gboolean
@@ -135,23 +159,77 @@ clutter_stage_cogl_realize (ClutterStageWindow *stage_window)
    * will be ignored, so we need to make sure the stage size is
    * updated to this size. */
 
-  if (cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_SWAP_BUFFERS_EVENT))
-    {
-      stage_cogl->swap_callback_id =
-        cogl_onscreen_add_swap_buffers_callback (stage_cogl->onscreen,
-                                                 handle_swap_complete_cb,
-                                                 stage_cogl);
-    }
-
+  stage_cogl->frame_closure =
+          cogl_onscreen_add_frame_callback (stage_cogl->onscreen,
+                                            frame_cb,
+                                            stage_cogl,
+                                            NULL);
   return TRUE;
 }
 
-static int
-clutter_stage_cogl_get_pending_swaps (ClutterStageWindow *stage_window)
+static void
+clutter_stage_cogl_schedule_update (ClutterStageWindow *stage_window,
+                                    gint                sync_delay)
+{
+  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+  gint64 now;
+  float refresh_rate;
+  gint64 refresh_interval;
+
+  if (stage_cogl->update_time != -1)
+    return;
+
+  now = g_get_monotonic_time ();
+
+  if (sync_delay < 0)
+    {
+      stage_cogl->update_time = now;
+      return;
+    }
+
+  /* We only extrapolate presentation times for 150ms  - this is somewhat
+   * arbitrary. The reasons it might not be accurate for larger times are
+   * that the refresh interval might be wrong or the vertical refresh
+   * might be downclocked if nothing is going on onscreen.
+   */
+  if (stage_cogl->last_presentation_time == 0||
+      stage_cogl->last_presentation_time < now - 150000)
+    {
+      stage_cogl->update_time = now;
+      return;
+    }
+
+  refresh_rate = stage_cogl->refresh_rate;
+  if (refresh_rate == 0.0)
+    refresh_rate = 60.0;
+
+  refresh_interval = (gint64) (0.5 + 1000000 / refresh_rate);
+  if (refresh_interval == 0)
+    refresh_interval = 16667; /* 1/60th second */
+
+  stage_cogl->update_time = stage_cogl->last_presentation_time + 1000 * sync_delay;
+
+  while (stage_cogl->update_time < now)
+    stage_cogl->update_time += refresh_interval;
+}
+
+static gint64
+clutter_stage_cogl_get_update_time (ClutterStageWindow *stage_window)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
 
-  return stage_cogl->pending_swaps;
+  if (stage_cogl->pending_swaps)
+    return -1; /* in the future, indefinite */
+
+  return stage_cogl->update_time;
+}
+
+static void
+clutter_stage_cogl_clear_update_time (ClutterStageWindow *stage_window)
+{
+  ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+
+  stage_cogl->update_time = -1;
 }
 
 static ClutterActor *
@@ -320,7 +398,10 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
   gboolean may_use_clipped_redraw;
   gboolean use_clipped_redraw;
   gboolean can_blit_sub_buffer;
+  gboolean has_buffer_age;
   ClutterActor *wrapper;
+  cairo_rectangle_int_t *clip_region;
+  gboolean force_swap;
 
   CLUTTER_STATIC_TIMER (painting_timer,
                         "Redrawing", /* parent */
@@ -348,6 +429,8 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
   can_blit_sub_buffer =
     cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_SWAP_REGION);
 
+  has_buffer_age = cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_BUFFER_AGE);
+
   may_use_clipped_redraw = FALSE;
   if (_clutter_stage_window_can_clip_redraws (stage_window) &&
       can_blit_sub_buffer &&
@@ -358,6 +441,7 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
       stage_cogl->frame_count > 3)
     {
       may_use_clipped_redraw = TRUE;
+      clip_region = &stage_cogl->bounding_redraw_clip;
     }
 
   if (may_use_clipped_redraw &&
@@ -367,23 +451,83 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
   else
     use_clipped_redraw = FALSE;
 
+  force_swap = FALSE;
+
+  if (use_clipped_redraw)
+    {
+      if (has_buffer_age)
+      {
+        int age = cogl_onscreen_get_buffer_age (stage_cogl->onscreen);
+        cairo_rectangle_int_t *current_damage;
+
+        current_damage = g_new0 (cairo_rectangle_int_t, 1);
+        current_damage->x = clip_region->x;
+        current_damage->y = clip_region->y;
+        current_damage->width = clip_region->width;
+        current_damage->height = clip_region->height;
+
+        stage_cogl->damage_history = g_slist_prepend (stage_cogl->damage_history, current_damage);
+
+        if (age != 0 && !stage_cogl->dirty_backbuffer && g_slist_length (stage_cogl->damage_history) >= age)
+          {
+            int i = 0;
+            GSList *tmp = NULL;
+            for (tmp = stage_cogl->damage_history; tmp; tmp = tmp->next)
+              {
+                _clutter_util_rectangle_union (clip_region, tmp->data, clip_region);
+                i++;
+                if (i == age)
+                  {
+                    g_slist_free_full (tmp->next, g_free);
+                    tmp->next = NULL;
+                  }
+              }
+
+            force_swap = TRUE;
+
+            CLUTTER_NOTE (CLIPPING, "Reusing back buffer - repairing region: x=%d, y=%d, width=%d, height=%d\n",
+                    clip_region->x,
+                    clip_region->y,
+                    clip_region->width,
+                    clip_region->height);
+
+          }
+        else if (age == 0 || stage_cogl->dirty_backbuffer)
+          {
+            CLUTTER_NOTE (CLIPPING, "Invalid back buffer: Resetting damage history list.\n");
+            g_slist_free_full (stage_cogl->damage_history, g_free);
+            stage_cogl->damage_history = NULL;
+          }
+
+      }
+    }
+  else
+    {
+      CLUTTER_NOTE (CLIPPING, "Unclipped redraw: Resetting damage history list.\n");
+      g_slist_free_full (stage_cogl->damage_history, g_free);
+      stage_cogl->damage_history = NULL;
+    }
+
+  if (has_buffer_age && !force_swap)
+    use_clipped_redraw = FALSE;
+
   if (use_clipped_redraw)
     {
       CLUTTER_NOTE (CLIPPING,
                     "Stage clip pushed: x=%d, y=%d, width=%d, height=%d\n",
-                    stage_cogl->bounding_redraw_clip.x,
-                    stage_cogl->bounding_redraw_clip.y,
-                    stage_cogl->bounding_redraw_clip.width,
-                    stage_cogl->bounding_redraw_clip.height);
+                    clip_region->x,
+                    clip_region->y,
+                    clip_region->width,
+                    clip_region->height);
 
       stage_cogl->using_clipped_redraw = TRUE;
 
-      cogl_clip_push_window_rectangle (stage_cogl->bounding_redraw_clip.x,
-                                       stage_cogl->bounding_redraw_clip.y,
-                                       stage_cogl->bounding_redraw_clip.width,
-                                       stage_cogl->bounding_redraw_clip.height);
+      cogl_clip_push_window_rectangle (clip_region->x,
+                                       clip_region->y,
+                                       clip_region->width,
+                                       clip_region->height);
       _clutter_stage_do_paint (CLUTTER_STAGE (wrapper),
-                               &stage_cogl->bounding_redraw_clip);
+                               clip_region);
       cogl_clip_pop ();
 
       stage_cogl->using_clipped_redraw = FALSE;
@@ -399,7 +543,7 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
           may_use_clipped_redraw)
         {
           _clutter_stage_do_paint (CLUTTER_STAGE (wrapper),
-                                   &stage_cogl->bounding_redraw_clip);
+                                   clip_region);
         }
       else
         _clutter_stage_do_paint (CLUTTER_STAGE (wrapper), NULL);
@@ -451,9 +595,9 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
   CLUTTER_TIMER_STOP (_clutter_uprof_context, painting_timer);
 
   /* push on the screen */
-  if (use_clipped_redraw)
+  if (use_clipped_redraw && !force_swap)
     {
-      cairo_rectangle_int_t *clip = &stage_cogl->bounding_redraw_clip;
+      cairo_rectangle_int_t *clip = clip_region;
       int copy_area[4];
 
       /* XXX: It seems there will be a race here in that the stage
@@ -503,6 +647,9 @@ clutter_stage_cogl_redraw (ClutterStageWindow *stage_window)
   /* reset the redraw clipping for the next paint... */
   stage_cogl->initialized_redraw_clip = FALSE;
 
+  /* We have repaired the backbuffer */
+  stage_cogl->dirty_backbuffer = FALSE;
+
   stage_cogl->frame_count++;
 }
 
@@ -514,6 +661,34 @@ clutter_stage_cogl_get_active_framebuffer (ClutterStageWindow *stage_window)
   return COGL_FRAMEBUFFER (stage_cogl->onscreen);
 }
 
+static void
+clutter_stage_cogl_dirty_back_buffer (ClutterStageWindow *stage_window)
+{
+ ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+
+ stage_cogl->dirty_backbuffer = TRUE;
+}
+
+static void
+clutter_stage_cogl_get_dirty_pixel (ClutterStageWindow *stage_window,
+                                    int *x, int *y)
+{
+    ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+    gboolean has_buffer_age = cogl_clutter_winsys_has_feature (COGL_WINSYS_FEATURE_BUFFER_AGE);
+    if ((stage_cogl->damage_history == NULL && has_buffer_age) || !has_buffer_age)
+      {
+        *x = 0;
+        *y = 0;
+      }
+    else
+     {
+        cairo_rectangle_int_t *rect;
+        rect = (cairo_rectangle_int_t *) (stage_cogl->damage_history->data);
+        *x = rect->x;
+        *y = rect->y;
+     }
+}
+
 static void
 clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
 {
@@ -524,13 +699,17 @@ clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
   iface->resize = clutter_stage_cogl_resize;
   iface->show = clutter_stage_cogl_show;
   iface->hide = clutter_stage_cogl_hide;
-  iface->get_pending_swaps = clutter_stage_cogl_get_pending_swaps;
+  iface->schedule_update = clutter_stage_cogl_schedule_update;
+  iface->get_update_time = clutter_stage_cogl_get_update_time;
+  iface->clear_update_time = clutter_stage_cogl_clear_update_time;
   iface->add_redraw_clip = clutter_stage_cogl_add_redraw_clip;
   iface->has_redraw_clips = clutter_stage_cogl_has_redraw_clips;
   iface->ignoring_redraw_clips = clutter_stage_cogl_ignoring_redraw_clips;
   iface->get_redraw_clip_bounds = clutter_stage_cogl_get_redraw_clip_bounds;
   iface->redraw = clutter_stage_cogl_redraw;
   iface->get_active_framebuffer = clutter_stage_cogl_get_active_framebuffer;
+  iface->dirty_back_buffer = clutter_stage_cogl_dirty_back_buffer;
+  iface->get_dirty_pixel = clutter_stage_cogl_get_dirty_pixel;
 }
 
 static void
@@ -571,4 +750,8 @@ _clutter_stage_cogl_class_init (ClutterStageCoglClass *klass)
 static void
 _clutter_stage_cogl_init (ClutterStageCogl *stage)
 {
+  stage->last_presentation_time = 0;
+  stage->refresh_rate = 0.0;
+
+  stage->update_time = -1;
 }
