@@ -47,6 +47,10 @@
 #include "config.h"
 #endif
 
+/* sadly, we are still using ClutterShader internally */
+#define COGL_DISABLE_DEPRECATION_WARNINGS
+#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
+
 /* This file depends on the glib enum types which aren't exposed
  * by cogl.h when COGL_ENABLE_EXPERIMENTAL_2_0_API is defined.
  *
@@ -58,9 +62,6 @@
 
 #define CLUTTER_ENABLE_EXPERIMENTAL_API
 
-/* sadly, we are still using ClutterShader internally */
-#define CLUTTER_DISABLE_DEPRECATION_WARNINGS
-
 #include "clutter-texture.h"
 
 #include "clutter-actor-private.h"
@@ -78,16 +79,6 @@
 #include "deprecated/clutter-texture.h"
 #include "deprecated/clutter-util.h"
 
-static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (ClutterTexture,
-                         clutter_texture,
-                         CLUTTER_TYPE_ACTOR,
-                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
-                                                clutter_scriptable_iface_init));
-
-#define CLUTTER_TEXTURE_GET_PRIVATE(obj)        (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_TEXTURE, ClutterTexturePrivate))
-
 typedef struct _ClutterTextureAsyncData ClutterTextureAsyncData;
 
 struct _ClutterTexturePrivate
@@ -194,6 +185,16 @@ static CoglPipeline *texture_template_pipeline = NULL;
 static void
 texture_fbo_free_resources (ClutterTexture *texture);
 
+static void clutter_scriptable_iface_init (ClutterScriptableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterTexture,
+                         clutter_texture,
+                         CLUTTER_TYPE_ACTOR,
+                         G_ADD_PRIVATE (ClutterTexture)
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_iface_init));
+
+
 GQuark
 clutter_texture_error_quark (void)
 {
@@ -972,8 +973,6 @@ clutter_texture_class_init (ClutterTextureClass *klass)
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
   GParamSpec *pspec;
 
-  g_type_class_add_private (klass, sizeof (ClutterTexturePrivate));
-
   actor_class->paint            = clutter_texture_paint;
   actor_class->pick             = clutter_texture_pick;
   actor_class->get_paint_volume = clutter_texture_get_paint_volume;
@@ -1296,7 +1295,7 @@ clutter_texture_init (ClutterTexture *self)
 {
   ClutterTexturePrivate *priv;
 
-  self->priv = priv = CLUTTER_TEXTURE_GET_PRIVATE (self);
+  self->priv = priv = clutter_texture_get_instance_private (self);
 
   priv->repeat_x          = FALSE;
   priv->repeat_y          = FALSE;
