@@ -134,9 +134,13 @@ clutter_stage_wayland_show (ClutterStageWindow *stage_window,
                             gboolean            do_raise)
 {
   ClutterStageCogl *stage_cogl = CLUTTER_STAGE_COGL (stage_window);
+  ClutterStageWayland *stage_wayland = CLUTTER_STAGE_WAYLAND (stage_window);
 
   clutter_stage_window_parent_iface->show (stage_window, do_raise);
 
+  /* TODO: must not call this on foreign surfaces when we add that support */
+  wl_shell_surface_set_toplevel (stage_wayland->wayland_shell_surface);
+
   /* We need to queue a redraw after the stage is shown because all of
    * the other queue redraws up to this point will have been ignored
    * because the actor was not visible. The other backends do not need
@@ -231,7 +235,6 @@ clutter_stage_wayland_class_init (ClutterStageWaylandClass *klass)
 
 /**
  * clutter_wayland_stage_get_wl_shell_surface: (skip)
- *
  * @stage: a #ClutterStage
  *
  * Access the underlying data structure representing the shell surface that is
@@ -240,7 +243,7 @@ clutter_stage_wayland_class_init (ClutterStageWaylandClass *klass)
  * Note: this function can only be called when running on the Wayland
  * platform. Calling this function at any other time will return %NULL.
  *
- * Returns: (transfer non): the Wayland shell surface associated with
+ * Returns: (transfer none): the Wayland shell surface associated with
  * @stage
  *
  * Since: 1.10
@@ -261,7 +264,6 @@ clutter_wayland_stage_get_wl_shell_surface (ClutterStage *stage)
 
 /**
  * clutter_wayland_stage_get_wl_surface: (skip)
- *
  * @stage: a #ClutterStage
  *
  * Access the underlying data structure representing the surface that is
@@ -271,6 +273,8 @@ clutter_wayland_stage_get_wl_shell_surface (ClutterStage *stage)
  * platform. Calling this function at any other time will return %NULL.
  *
  * Returns: (transfer none): the Wayland surface associated with @stage
+ *
+ * Since: 1.10
  */
 struct wl_surface *
 clutter_wayland_stage_get_wl_surface (ClutterStage *stage)
